package com.bankaudit.dao;

import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.hibernate.Query;
import org.hibernate.ScrollMode;
import org.hibernate.ScrollableResults;
import org.hibernate.Session;
import org.hibernate.transform.ResultTransformer;
import org.jboss.logging.Logger;
import org.springframework.stereotype.Repository;

import com.bankaudit.constants.BankAuditConstant;
import com.bankaudit.dto.DataTableResponse;
import com.bankaudit.model.MaintLossEvents;
import com.bankaudit.util.BankAuditUtil;

import java.util.ArrayList;

@Repository("maintLossEventsDao")
public class MaintLossEventsDaoImpl extends AbstractDao implements MaintLossEventsDao {
	
	static final Logger logger = Logger.getLogger(MaintLossEventsDaoImpl.class);
	
	@Override
	public DataTableResponse getAllMaintLossEvents(String search, Integer orderColumn, String orderDirection,
			Integer page, Integer size,Integer legalEntityCode) {
		
		
	    Session session = getSession();
	    session.setDefaultReadOnly(true);
		List<MaintLossEvents> maintLossEvents = null;
		
		DataTableResponse dataTableResponse=new DataTableResponse();
		StringBuilder queryString=new StringBuilder(
				
				         " SELECT "
						 + "  "
						 + " legal_entity_code, "
						 + " loss_event_code, "
						 + " loss_event_type,  "
						 + " loss_event_desc, "
						  + " auth_rej_remarks, "

				         + " (select concat(gp.value , '-' , gp.description) from general_parameter gp where"
				         + " gp.key_1='ENTITY_STATUS' and gp.key_2='ENTITY_STATUS' and gp.value=c.entity_status "
				         + " and gp.legal_entity_code=c.legal_entity_code ) as entity_status ,  "
						
						 + " (select concat(gp.value , '-' , gp.description)"
						 + " from general_parameter gp where "
						 + " gp.key_1='STATUS' and gp.value=c.status "
						 + " and gp.legal_entity_code=c.legal_entity_code ) as status, "
						 
						 + "( select concat(u.user_id , '-' ,u.first_name,' ',u.last_name )"
						 + " from user u where "
						 + " u.user_id=c.maker "
						 + " and u.legal_entity_code=c.legal_entity_code ) as maker ,  "
						 
						 + " maker_timestamp, "
						 + " checker, "
						 + " checker_timestamp, "
						 + " businessdate_timestamp, "
						 + " sysdate_timestamp "
						 + "       "
						 + "     FROM "
						 + "         ( SELECT "
						 + "             *  "
						 + "         FROM "
						 + "             maint_loss_events "
						 + "         WHERE "
						 + "             legal_entity_code =:legal_entity_code "
						 + "         UNION ALL "
						 + "         SELECT "
						 + "             *  "
						 + "         FROM "
						 + "            maint_loss_events_wrk  "
						 + "         WHERE "
						 + "             legal_entity_code =:legal_entity_code "
						 + "              "
						 + "     ) AS c  "
						 + " WHERE "
						 + "    legal_entity_code =:legal_entity_code ");
		
		if(!BankAuditUtil.isEmptyString(search)){
			queryString=queryString.append(" and ( c.loss_event_code like :search "
					+ " or  c.loss_event_type like :search  "
					+ " or  c.loss_event_desc like :search  "
					+ " or  c.maker like :search  "
					+ " or  c.status IN (SELECT value FROM general_parameter WHERE key_1='STATUS' and  description LIKE :search ) "
					+ " or  c.entity_status IN (SELECT value FROM general_parameter  WHERE key_1='ENTITY_STATUS' and key_2='ENTITY_STATUS' and description LIKE :search ) "
                    + " or  c.maker IN ( "
					+ " select u.user_id "
						 + " from user u where "
						 + " (concat(u.first_name , ' ' , u.last_name) like :search  ) "
						 + " and u.legal_entity_code =:legal_entity_code ) "
					+ " ) "
                    
					);
		}
		
	
		String[] columns={
				"c.loss_event_code",
				"c.loss_event_type",
				"c.loss_event_desc",
				"c.maker",
				"c.entity_status",
				"c.status"};
		
		if(orderColumn != null && orderColumn == -1 && !BankAuditUtil.isEmptyString(orderDirection)){ // -1 send for the first time from the UI during landing page.
			queryString=queryString.append(" order by FIELD(STATUS,'DF','R','RM','U','M','A') ASC  "); // The code modified to get the records in the order of Status on landing page, the sequence is  Draft, Reject, Unauth, Modify, Auth
		}else if(orderColumn!=null && !BankAuditUtil.isEmptyString(orderDirection)){
			queryString=queryString.append(" order by ").append(columns[orderColumn]).append(" ").append(orderDirection);
		}
		
		Query query=session.createSQLQuery(queryString.toString())
				.setResultTransformer(new ResultTransformer() {
					
					@Override
					public Object transformTuple(Object[] tuple, String[] aliases) {
						
						MaintLossEvents maintLossEvents=null;
						
						if(tuple!=null){
							
							maintLossEvents=new MaintLossEvents();						
							if(tuple[0]!=null){
								maintLossEvents.setLegalEntityCode(Integer.parseInt(tuple[0].toString()));
							}
							if(tuple[1]!=null){
								maintLossEvents.setLossEventCode(tuple[1].toString());
							}
							if(tuple[2]!=null){
								maintLossEvents.setLossEventType(tuple[2].toString());
							}
							if(tuple[3]!=null){
								maintLossEvents.setLossEventDesc(tuple[3].toString());
							}
							if(tuple[4]!=null){
								maintLossEvents.setAuthRejRemarks(tuple[4].toString());
							}
							if(tuple[5]!=null){
								maintLossEvents.setEntityStatus(tuple[5].toString());
							}
							if(tuple[6]!=null){
								maintLossEvents.setStatus(tuple[6].toString());
							}
							if(tuple[7]!=null){
								maintLossEvents.setMaker(tuple[7].toString());
							}
							if(tuple[8]!=null){
								maintLossEvents.setMakerTimestamp((Date)tuple[8]);
							}
							if(tuple[9]!=null){
								maintLossEvents.setChecker(tuple[9].toString());
							}
							if(tuple[10]!=null){
								maintLossEvents.setCheckerTimestamp((Date)tuple[10]);
							}
							if(tuple[11]!=null){
								maintLossEvents.setBusinessdateTimestamp((Date)tuple[11]);
							}
							if(tuple[12]!=null){
								maintLossEvents.setSysdateTimestamp((Date)tuple[12]);
							}
							
						}
						
						return maintLossEvents;
					}
					
					@Override
					public List transformList(List collection) {
						return collection;
					}
				})
				.setParameter("legal_entity_code", legalEntityCode);
		
	
		
		if(!BankAuditUtil.isEmptyString(search)){
			query.setParameter("search", "%"+search+"%");
		}
		
		
		ScrollableResults resultScroll = query.scroll(ScrollMode.SCROLL_INSENSITIVE);
		if(resultScroll.first()&&resultScroll.scroll(page*size)){
			
			maintLossEvents= new ArrayList<>();
			int i = 0;
			while (size > i++) {
				maintLossEvents.add((MaintLossEvents) resultScroll.get(0));
			    if (!resultScroll.next())
			        break;
			}
			dataTableResponse.setData(maintLossEvents);
			resultScroll.last();
			if(size!=0){
				dataTableResponse.setRecordsFiltered((resultScroll.getRowNumber()+1l/size)+1);
				dataTableResponse.setRecordsTotal((resultScroll.getRowNumber()+1l/size)+1);
			}else{
				dataTableResponse.setError("page size zero");
				dataTableResponse.setData(Collections.EMPTY_LIST);
				dataTableResponse.setRecordsTotal(0l);
				dataTableResponse.setRecordsFiltered(0l);
			}
			
		}else{
			dataTableResponse.setError(null);
			dataTableResponse.setData(Collections.EMPTY_LIST);
			dataTableResponse.setRecordsTotal(0l);
			dataTableResponse.setRecordsFiltered(0l);
		}
		return dataTableResponse;
	}
	
	
		@Override
		public void deleteMaintLossEvents(Integer legalEntityCode, String lossEventCode, String lossEventType,
				String statusAuth) {

			Session session=getSession();
				
			if(!BankAuditConstant.STATUS_AUTH.equals(statusAuth)){
				session.createQuery("delete from MaintLossEventsWrk where legalEntityCode =:legalEntityCode  and "
						+ " lossEventCode =:lossEventCode "
						+ " and lossEventType =:lossEventType  ")
				.setParameter("lossEventType", lossEventType)
				.setParameter("lossEventCode", lossEventCode)
				.setParameter("legalEntityCode", legalEntityCode).executeUpdate();
				
			}else {

				session.createQuery("delete from MaintLossEvents where legalEntityCode =:legalEntityCode  and "
						+ " lossEventCode =:lossEventCode "
						+ " and lossEventType =:lossEventType  ")
				.setParameter("lossEventType", lossEventType)
				.setParameter("lossEventCode", lossEventCode)
				.setParameter("legalEntityCode", legalEntityCode).executeUpdate();
			}
			
		}
		
	@Override
	public Boolean isMaintLossEvents(Integer legalEntityCode, String lossEventCode, String lossEventType) {
		Session session=getSession();
		Long count=(Long)session.createQuery(
				" select count(*) "
				+ " from MaintLossEvents   "
				+ " where "
				+ " legalEntityCode =:legalEntityCode  "
				+ " and lossEventType =:lossEventType "
				+ " and lossEventCode =:lossEventCode  ")
		.setParameter("legalEntityCode", legalEntityCode)
		.setParameter("lossEventType", lossEventType)
		.setParameter("lossEventCode", lossEventCode).uniqueResult();
		
		Long count1=(Long)session.createQuery(
		
				
				" select count(*) "
				+ " from MaintLossEventsWrk   "
				+ " where "
				+ " legalEntityCode =:legalEntityCode "
				+ " and lossEventType =:lossEventType "
				+ " and lossEventCode =:lossEventCode  ")
		.setParameter("legalEntityCode", legalEntityCode)
		.setParameter("lossEventType", lossEventType)
		.setParameter("lossEventCode", lossEventCode).uniqueResult();
		Boolean flag=false;
		if((count1!=null && count1 >0)||(count!=null && count >0)){
			flag= true;
		}else {
			flag= false;
		}
		return flag;
	}
}
