package com.bankaudit.dao;

import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.hibernate.Query;
import org.hibernate.ScrollMode;
import org.hibernate.ScrollableResults;
import org.hibernate.Session;
import org.hibernate.transform.ResultTransformer;
import org.jboss.logging.Logger;
import org.springframework.stereotype.Repository;

import com.bankaudit.constants.BankAuditConstant;
import com.bankaudit.dto.DataTableResponse;
import com.bankaudit.model.MaintSeverityScale;
import com.bankaudit.util.BankAuditUtil;

import java.util.ArrayList;

@Repository("maintSeverityScaleDao")
public class MaintSeverityScaleDaoImpl extends AbstractDao implements MaintSeverityScaleDao  {
	
	static final Logger logger = Logger.getLogger(MaintSeverityScaleDaoImpl.class);
	
	@Override
	public DataTableResponse getAllMaintSeverityScale(String search, Integer orderColumn, String orderDirection,
			Integer page, Integer size,Integer legalEntityCode) {
		
		
	    Session session = getSession();
	    session.setDefaultReadOnly(true);
		List<MaintSeverityScale> maintSeverityScales = null;
		
		DataTableResponse dataTableResponse=new DataTableResponse();
		StringBuilder queryString=new StringBuilder(
				
				         " SELECT "
						 + "  "
						 + " legal_entity_code, "
						 + " severity_scale_code, "
						 + " severity_scale_type,  "
						 + " severity_scale_desc, "
						 + " severity_scale_score, "
						 + " auth_rej_remarks, "

				         + " (select concat(gp.value , '-' , gp.description) from general_parameter gp where"
				         + " gp.key_1='ENTITY_STATUS' and gp.key_2='ENTITY_STATUS' and gp.value=c.entity_status "
				         + " and gp.legal_entity_code=c.legal_entity_code ) as entity_status ,  "
						
						 + " (select concat(gp.value , '-' , gp.description)"
						 + " from general_parameter gp where "
						 + " gp.key_1='STATUS' and gp.value=c.status "
						 + " and gp.legal_entity_code=c.legal_entity_code ) as status, "
						 
						 + "( select concat(u.user_id , '-' ,u.first_name,' ',u.last_name )"
						 + " from user u where "
						 + " u.user_id=c.maker "
						 + " and u.legal_entity_code=c.legal_entity_code ) as maker ,  "
						 
						 + " maker_timestamp, "
						 + " checker, "
						 + " checker_timestamp, "
						 + " businessdate_timestamp, "
						 + " sysdate_timestamp "
						 + "       "
						 + "     FROM "
						 + "         ( SELECT "
						 + "             *  "
						 + "         FROM "
						 + "             maint_severity_scale "
						 + "         WHERE "
						 + "             legal_entity_code =:legal_entity_code "
						 + "         UNION ALL "
						 + "         SELECT "
						 + "             *  "
						 + "         FROM "
						 + "            maint_severity_scale_wrk  "
						 + "         WHERE "
						 + "             legal_entity_code =:legal_entity_code "
						 + "              "
						 + "     ) AS c  "
						 + " WHERE "
						 + "    legal_entity_code =:legal_entity_code ");
		
		if(!BankAuditUtil.isEmptyString(search)){
			queryString=queryString.append(" and ( c.severity_scale_code like :search "
					+ " or  c.severity_scale_type like :search  "
					+ " or  c.severity_scale_desc like :search  "
					+ " or  c.maker like :search  "
					+ " or  c.status IN (SELECT value FROM general_parameter WHERE key_1='STATUS' and  description LIKE :search ) "
					+ " or  c.severity_scale_score  like :search  "
					+ " or  c.entity_status IN (SELECT value FROM general_parameter  WHERE key_1='ENTITY_STATUS' and key_2='ENTITY_STATUS' and description LIKE :search ) "
                    + " or  c.maker IN ( "
					+ " select u.user_id "
						 + " from user u where "
						 + " (concat(u.first_name , ' ' , u.last_name) like :search  ) "
						 + " and u.legal_entity_code =:legal_entity_code ) "
					+ " ) "
                    
					);
		}
		
	
		String[] columns={
				"c.severity_scale_code",
				"c.severity_scale_type",
				"c.severity_scale_desc",
				"c.severity_scale_score",
				"c.maker",
				"c.entity_status",
				"c.status"};
		
		if(orderColumn != null && orderColumn == -1 && !BankAuditUtil.isEmptyString(orderDirection)){ // -1 send for the first time from the UI during landing page.
			queryString=queryString.append(" order by FIELD(STATUS,'DF','R','RM','U','M','A') ASC  "); // The code modified to get the records in the order of Status on landing page, the sequence is  Draft, Reject, Unauth, Modify, Auth
		}else if(orderColumn!=null && !BankAuditUtil.isEmptyString(orderDirection)){
			queryString=queryString.append(" order by ").append(columns[orderColumn]).append(" ").append(orderDirection);
		}
		
		Query query=session.createSQLQuery(queryString.toString())
				.setResultTransformer(new ResultTransformer() {
					
					@Override
					public Object transformTuple(Object[] tuple, String[] aliases) {
						
						MaintSeverityScale maintSeverityScale=null;
						
						if(tuple!=null){
							
							maintSeverityScale=new MaintSeverityScale();
							
							if(tuple[0]!=null){
								maintSeverityScale.setLegalEntityCode(Integer.parseInt(tuple[0].toString()));
							}
							if(tuple[1]!=null){
								maintSeverityScale.setSeverityScaleCode(tuple[1].toString());
							}
							if(tuple[2]!=null){
								maintSeverityScale.setSeverityScaleType(tuple[2].toString());
							}
							if(tuple[3]!=null){
								maintSeverityScale.setSeverityScaleDesc(tuple[3].toString());
							}
							if(tuple[4]!=null){
								maintSeverityScale.setSeverityScaleScore(tuple[4].toString());
							}
							if(tuple[5]!=null){
								maintSeverityScale.setAuthRejRemarks(tuple[5].toString());
							}
							if(tuple[6]!=null){
								maintSeverityScale.setEntityStatus(tuple[6].toString());
							}
							if(tuple[7]!=null){
								maintSeverityScale.setStatus(tuple[7].toString());
							}
							if(tuple[8]!=null){
								maintSeverityScale.setMaker(tuple[8].toString());
							}
							if(tuple[9]!=null){
								maintSeverityScale.setMakerTimestamp((Date)tuple[9]);
							}
							if(tuple[10]!=null){
								maintSeverityScale.setChecker(tuple[10].toString());
							}
							if(tuple[11]!=null){
								maintSeverityScale.setCheckerTimestamp((Date)tuple[11]);
							}
							if(tuple[12]!=null){
								maintSeverityScale.setBusinessdateTimestamp((Date)tuple[12]);
							}
							if(tuple[13]!=null){
								maintSeverityScale.setSysdateTimestamp((Date)tuple[13]);
							}
							
						}
						
						return maintSeverityScale;
					}
					
					@Override
					public List transformList(List collection) {
						return collection;
					}
				})
				.setParameter("legal_entity_code", legalEntityCode);
		
	
		
		if(!BankAuditUtil.isEmptyString(search)){
			query.setParameter("search", "%"+search+"%");
		}
		
		
		ScrollableResults resultScroll = query.scroll(ScrollMode.SCROLL_INSENSITIVE);
		if(resultScroll.first()&&resultScroll.scroll(page*size)){
			
			maintSeverityScales= new ArrayList<>();
			int i = 0;
			while (size > i++) {
				maintSeverityScales.add((MaintSeverityScale) resultScroll.get(0));
			    if (!resultScroll.next())
			        break;
			}
			dataTableResponse.setData(maintSeverityScales);
			resultScroll.last();
			if(size!=0){
				dataTableResponse.setRecordsFiltered((resultScroll.getRowNumber()+1l/size)+1);
				dataTableResponse.setRecordsTotal((resultScroll.getRowNumber()+1l/size)+1);
			}else{
				dataTableResponse.setError("page size zero");
				dataTableResponse.setData(Collections.EMPTY_LIST);
				dataTableResponse.setRecordsTotal(0l);
				dataTableResponse.setRecordsFiltered(0l);
			}
			
		}else{
			dataTableResponse.setError(null);
			dataTableResponse.setData(Collections.EMPTY_LIST);
			dataTableResponse.setRecordsTotal(0l);
			dataTableResponse.setRecordsFiltered(0l);
		}
		return dataTableResponse;
	}
		
		@Override
		public void deleteMaintSeverityScale(Integer legalEntityCode, String severityScaleCode, String severityScaleType,
				String statusAuth) {

			
			Session session=getSession();
				
			if(!BankAuditConstant.STATUS_AUTH.equals(statusAuth)){
				session.createQuery("delete from MaintSeverityScaleWrk where legalEntityCode =:legalEntityCode  and "
						+ " severityScaleCode =:severityScaleCode "
						+ " and severityScaleType =:severityScaleType  ")
				.setParameter("severityScaleType", severityScaleType)
				.setParameter("severityScaleCode", severityScaleCode)
				.setParameter("legalEntityCode", legalEntityCode).executeUpdate();
				
			}else {

				session.createQuery("delete from MaintSeverityScale where legalEntityCode =:legalEntityCode  and "
						+ " severityScaleCode =:severityScaleCode "
						+ " and severityScaleType =:severityScaleType  ")
				.setParameter("severityScaleType", severityScaleType)
				.setParameter("severityScaleCode", severityScaleCode)
				.setParameter("legalEntityCode", legalEntityCode).executeUpdate();
			}
			
		}

		@Override
		public Boolean isMaintSeverityScale(Integer legalEntityCode, String severityScaleCode, String severityScaleType) {
			Session session=getSession();
			Long count=(Long)session.createQuery(
					" select count(*) "
					+ " from MaintSeverityScale   "
					+ " where "
					+ " legalEntityCode =:legalEntityCode  "
					+ " and severityScaleType =:severityScaleType "
					+ " and severityScaleCode =:severityScaleCode  ")
			.setParameter("legalEntityCode", legalEntityCode)
			.setParameter("severityScaleType", severityScaleType)
			.setParameter("severityScaleCode", severityScaleCode).uniqueResult();
			
			Long count1=(Long)session.createQuery(
			
					
					" select count(*) "
					+ " from MaintSeverityScaleWrk   "
					+ " where "
					+ " legalEntityCode =:legalEntityCode "
					+ " and severityScaleType =:severityScaleType "
					+ " and severityScaleCode =:severityScaleCode  ")
			.setParameter("legalEntityCode", legalEntityCode)
			.setParameter("severityScaleType", severityScaleType)
			.setParameter("severityScaleCode", severityScaleCode).uniqueResult();
			Boolean flag=false;
			if((count1!=null && count1 >0)||(count!=null && count >0)){
				flag= true;
			}else {
				flag= false;
			}
			return flag;
		}
		
		@Override
		public Boolean validateScore(MaintSeverityScale maintSeverityScale) {
			Session session=getSession();
			logger.info("LE::"+maintSeverityScale.getLegalEntityCode()+" ProbType::"+maintSeverityScale.getSeverityScaleType()+" Score::"+maintSeverityScale.getSeverityScaleScore()+" CriticalityCode::"+maintSeverityScale.getSeverityScaleCode());
			Long scoreMstCnt=(Long)session.createQuery(  
					" select count(*) "
					+ " from MaintSeverityScale   "
					+ " where "
					+ " legalEntityCode =:legalEntityCode  "
					+ " and severityScaleType =:severityScaleType "
					+ " and severityScaleCode <>:severityScaleCode "  // Condition added to ignore the score check for the same Criticality Code
					+ " and severityScaleScore =:severityScaleScore  ")
			.setParameter("legalEntityCode", maintSeverityScale.getLegalEntityCode())
			.setParameter("severityScaleType", maintSeverityScale.getSeverityScaleType())
			.setParameter("severityScaleCode", maintSeverityScale.getSeverityScaleCode())
			.setParameter("severityScaleScore", maintSeverityScale.getSeverityScaleScore()).uniqueResult();
			
			Long ScoreWrkCnt=(Long)session.createQuery(
					" select count(*) "
					+ " from MaintSeverityScaleWrk   "
					+ " where "
					+ " legalEntityCode =:legalEntityCode "
					+ " and severityScaleType =:severityScaleType "
					+ " and severityScaleCode <>:severityScaleCode " // Condition added to ignore the score check for the same Criticality Code
					+ " and severityScaleScore =:severityScaleScore  ")
					.setParameter("legalEntityCode", maintSeverityScale.getLegalEntityCode())
					.setParameter("severityScaleType", maintSeverityScale.getSeverityScaleType())
					.setParameter("severityScaleCode", maintSeverityScale.getSeverityScaleCode())
					.setParameter("severityScaleScore", maintSeverityScale.getSeverityScaleScore()).uniqueResult();
			
			logger.info("scoreMstCnt::"+scoreMstCnt +" ScoreWrkCnt::" +ScoreWrkCnt);
			Boolean flag=false;
			if((ScoreWrkCnt!=null && ScoreWrkCnt >0)||(scoreMstCnt!=null && scoreMstCnt >0)){
				flag= true;
			}
			return flag;
		}
}
