/*
 * 
 */
package com.bankaudit.rest.controller;

import java.sql.Timestamp;
import java.util.List;

import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;

import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.bankaudit.constants.BankAuditConstant;
import com.bankaudit.dto.DataTableResponse;
import com.bankaudit.model.MaintAuditProcess;
import com.bankaudit.model.MaintAuditProcess;
import com.bankaudit.service.BusinessDateService;
import com.bankaudit.service.MaintAuditProcessService;
import com.bankaudit.service.SequenceAppenderService;
import com.bankaudit.util.BankAuditUtil;
import com.bankaudit.util.ServiceStatus;


/**
 * The Class {@link MaintAuditProcessController} provides the REST services to
 * manage the {@link MaintAuditProcess} class objects .
 *
 * @author amit.patel
 * @version 1.0
 */


@RestController

@RequestMapping("/api/maintAuditProcess")
public class MaintAuditProcessController {

	/**
	 * The maint audit process service is autowired and make methods available
	 * from service layer .
	 */
	@Autowired
	MaintAuditProcessService maintAuditProcessService;

	/**
	 * The business date service is autowired and make methods available from
	 * service layer .
	 */
	@Autowired
	BusinessDateService businessDateService;

	/**
	 * The sequence appender service is autowired and make methods available
	 * from service layer .
	 */
	@Autowired
	SequenceAppenderService sequenceAppenderService;

	/** The Constant logger is used to specify the . */
	static final Logger logger = Logger.getLogger(MaintAuditProcessController.class);

	/**
	 * This method is use to Creates the maint audit process.
	 *
	 * @param maintAuditProcess
	 *            specify the maint audit process
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus createMaintAuditProcess(@RequestBody MaintAuditProcess maintAuditProcess) {
		ServiceStatus serviceStatus = new ServiceStatus();
		try {
			if (maintAuditProcess != null && maintAuditProcess.getLegalEntityCode() != null
					&& !BankAuditUtil.isEmptyString(maintAuditProcess.getProcessCode())
					&& !BankAuditUtil.isEmptyString(maintAuditProcess.getProcessName())
					&& !BankAuditUtil.isEmptyString(maintAuditProcess.getMaker())) {

				maintAuditProcess.setProcessId(sequenceAppenderService.getAutoSequenceId());
					maintAuditProcess.setBusinessdateTimestamp(businessDateService
							.getBusinessDateTimestampByLegalEntityCode(maintAuditProcess.getLegalEntityCode()));
					maintAuditProcess.setSysdateTimestamp(new Timestamp(System.currentTimeMillis()));
					maintAuditProcess.setMakerTimestamp(maintAuditProcess.getBusinessdateTimestamp());
					maintAuditProcessService.createMaintAuditProcess(maintAuditProcess);
	
					serviceStatus.setStatus("success");
					serviceStatus.setMessage("successfully created");
				}else {
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("invalid payload");
			}
		}catch (Exception e) {
			serviceStatus.setStatus("failure");
			if(e instanceof org.springframework.dao.DataIntegrityViolationException) {
				serviceStatus.setMessage("DATAINTGRTY_VIOLATION");
			}else serviceStatus.setMessage("EXCP_OCCURED");
		}

		return serviceStatus;
	}

	/**
	 * This method is use to Update maint audit process.
	 *
	 * @param maintAuditProcess
	 *            specify the maint audit process
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(method=RequestMethod.PUT,consumes=MediaType.APPLICATION_JSON_VALUE,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus updateMaintAuditProcess(@RequestBody MaintAuditProcess maintAuditProcess){
		ServiceStatus serviceStatus=new ServiceStatus();
		
		if(maintAuditProcess!=null
				&& maintAuditProcess.getLegalEntityCode()!=null
				&& !BankAuditUtil.isEmptyString(maintAuditProcess.getProcessName())
				&& !BankAuditUtil.isEmptyString(maintAuditProcess.getProcessCode())
				&& !BankAuditUtil.isEmptyString(maintAuditProcess.getMaker())
				){
			
			try {
				
			    maintAuditProcess.setBusinessdateTimestamp(businessDateService.getBusinessDateTimestampByLegalEntityCode(maintAuditProcess.getLegalEntityCode()));
			    maintAuditProcess.setSysdateTimestamp(new Timestamp(System.currentTimeMillis()));
				if(maintAuditProcess.getStatus().equals(BankAuditConstant.STATUS_AUTH)){
					maintAuditProcess.setCheckerTimestamp(businessDateService.getBusinessDateTimestampByLegalEntityCode(maintAuditProcess.getLegalEntityCode()));
				} else{
					maintAuditProcess.setMakerTimestamp(businessDateService.getBusinessDateTimestampByLegalEntityCode(maintAuditProcess.getLegalEntityCode()));
				}
			    maintAuditProcessService.updateMaintAuditProcess(maintAuditProcess);
			    
			    serviceStatus.setStatus("success");
			    serviceStatus.setMessage("successfully updated");
		
			} catch (Exception e) {
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("failure");
				if(e instanceof org.springframework.dao.DataIntegrityViolationException) {
					serviceStatus.setMessage("DATAINTGRTY_VIOLATION");
				}
			}
			
			
		}else {
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("invalid payload ");
		}
		
		return serviceStatus;
	}



	/**
	 * This method is use to Gets the maint audit process.
	 *
	 * @param drawStr
	 *            specify the draw str
	 * @param length
	 *            specify the length
	 * @param start
	 *            specify the start
	 * @param search
	 *            specify the search
	 * @param orderColumnStr
	 *            specify the order column str
	 * @param orderDirection
	 *            specify the order direction
	 * @param legalEntityCodeStr
	 *            specify the legal entity code str
	 * @return the data table response class object with response status and
	 *         payload .
	 */
	
@RequestMapping(value="/getAll",method=RequestMethod.GET,produces=MediaType.APPLICATION_JSON_VALUE) 
	DataTableResponse getMaintAuditProcess(
			@RequestParam("draw") String drawStr,
			@RequestParam("length") String length,
			@RequestParam("start") String start,
			@RequestParam("search[value]") String search,
			@RequestParam("order[0][column]") String orderColumnStr,
			@RequestParam("order[0][dir]")String orderDirection,
			@RequestParam("legalEntityCode")String legalEntityCodeStr){
		
		DataTableResponse dataTableResponse=null;
		
		if(!BankAuditUtil.isEmptyString(drawStr)
				&& !BankAuditUtil.isEmptyString(length)
				&& !BankAuditUtil.isEmptyString(start)
				&& !BankAuditUtil.isEmptyString(legalEntityCodeStr)){
			try {
				Integer legalEntityCode=Integer.parseInt(legalEntityCodeStr);
				Integer draw=Integer.parseInt(drawStr);
				Integer size=Integer.parseInt(length);
				Integer page=Integer.parseInt(start)/size;
				Integer orderColumn=null;
				if(!BankAuditUtil.isEmptyString(orderColumnStr)){
					orderColumn=Integer.parseInt(orderColumnStr);
				}
				dataTableResponse = maintAuditProcessService.getMaintAuditProcess(legalEntityCode,search,orderColumn,orderDirection,page,size);
				dataTableResponse.setDraw(draw);    
					
			} catch (Exception e) {
                dataTableResponse=new DataTableResponse();
                dataTableResponse.setError(e.getMessage());
				e.printStackTrace();
			}
		
		}else {
			dataTableResponse=new DataTableResponse();
			dataTableResponse.setError("invalid data");
		}
		
		return dataTableResponse;
	}

	
	/**
	 * This method is use to Gets the maint audit process.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param processCode
	 *            specify the process code
	 * @param status
	 *            specify the status
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(value="/getByLegalEntityCodeAndProcessCode/{legalEntityCode}/{processCode}/{status}",method=RequestMethod.GET,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus getMaintAuditProcess(@PathVariable("legalEntityCode")Integer legalEntityCode,
			@PathVariable("processCode") String processCode,
			@PathVariable("status") String status
			){
		ServiceStatus serviceStatus=new ServiceStatus();
		
		if(legalEntityCode!=null 
				&& !BankAuditUtil.isEmptyString(processCode)){
			
			try {
			    MaintAuditProcess maintAuditProcess=maintAuditProcessService.getMaintAuditProcess(legalEntityCode,processCode,status);
			    if(maintAuditProcess!=null){
			    	serviceStatus.setResult(maintAuditProcess);
				    serviceStatus.setStatus("success");
				    serviceStatus.setMessage("successfully retrieved ");	
			    }else {
					serviceStatus.setStatus("failure");
					serviceStatus.setMessage("maintAuditProcess not found");
				}
				
			} catch (Exception e) {
				e.printStackTrace();
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("failure");
			}
			
		}else {
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("invalid payload ");
		}
		
		return serviceStatus;
	}
	
	/**
	 * This method is use to Gets the maint audit process.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(value="/get/{legalEntityCode}",method=RequestMethod.GET,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus getMaintAuditProcess(@PathVariable("legalEntityCode")Integer legalEntityCode
			){
		ServiceStatus serviceStatus=new ServiceStatus();
		
		if(legalEntityCode!=null){
			
			try {
			    List<MaintAuditProcess> maintAuditProcesss=maintAuditProcessService.getMaintAuditProcess(legalEntityCode);
			    if(maintAuditProcesss!=null && !maintAuditProcesss.isEmpty()){
			    	serviceStatus.setResult(maintAuditProcesss);
				    serviceStatus.setStatus("success");
				    serviceStatus.setMessage("successfully retrieved ");	
			    }else {
					serviceStatus.setStatus("failure");
					serviceStatus.setMessage("maintAuditProcess not found");
				}
				
			} catch (Exception e) {
				e.printStackTrace();
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("failure");
			}
			
		}else {
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("invalid payload");
		}
		
		return serviceStatus;
	}
	
	/**
	 * This method is use to Checks if is maint audit process.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param processCode
	 *            specify the process code
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(value="/isMaintAuditProcess/{legalEntityCode}/{processCode}",method=RequestMethod.GET,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus isMaintAuditProcess(@PathVariable("legalEntityCode")Integer legalEntityCode,
			@PathVariable("processCode") String processCode
			){
		ServiceStatus serviceStatus=new ServiceStatus();
		
		if(legalEntityCode!=null 
				&& !BankAuditUtil.isEmptyString(processCode)){
			
			try {
			    Boolean maintAuditProcess=maintAuditProcessService.isMaintAuditProcess(legalEntityCode,processCode);
		    	serviceStatus.setResult(maintAuditProcess);
			    serviceStatus.setStatus("success");
			    serviceStatus.setMessage("successfully retrieved ");	
		    
				
			} catch (Exception e) {
				e.printStackTrace();
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("failure");
			}
			
		}else {
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("invalid payload ");
		}
		
		return serviceStatus;
	}
	
	// Service to delete the Records, 
		
@RequestMapping(value="/deleteProcessByID/{legalEntityCode}/{processId}/{userId}",method=RequestMethod.DELETE,produces=MediaType.APPLICATION_JSON_VALUE)
		ServiceStatus deleteProcessByID(@PathVariable("legalEntityCode")Integer legalEntityCode,
				@PathVariable("processId") String processId, @PathVariable("userId") String userId){
		ServiceStatus serviceStatus=new ServiceStatus();
			
			if(legalEntityCode !=null  && !BankAuditUtil.isEmptyString(processId)){
				try {
					maintAuditProcessService.deleteProcessByID(legalEntityCode, processId, userId);
					serviceStatus.setStatus("success");
					serviceStatus.setMessage("Record deleted successfully.");
				} catch (Exception e) {
					serviceStatus.setStatus("failure");
					e.printStackTrace();
					if(e instanceof org.springframework.dao.DataIntegrityViolationException) {
						serviceStatus.setMessage("DATAINTGRTY_VIOLATION");
					}
					serviceStatus.setMessage("failure");
				}
			}else {
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("invalid payload");
			}
			return serviceStatus;
		}	

}
