/*
 * 
 */
package com.bankaudit.rest.controller;

import java.sql.Timestamp;
import java.util.List;

import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.bankaudit.constants.BankAuditConstant;
import com.bankaudit.dto.DataTableResponse;
import com.bankaudit.model.MaintAuditTypeDesc;
import com.bankaudit.service.BusinessDateService;
import com.bankaudit.service.MaintAuditTypeDescService;
import com.bankaudit.service.SequenceAppenderService;
import com.bankaudit.util.BankAuditUtil;
import com.bankaudit.util.ServiceStatus;


/**
 * The Class {@link MaintAuditTypeDescController} provides the REST services to
 * manage the {@link MaintAuditTypeDesc} class objects .
 *
 * @author amit.patel
 * @version 1.0
 */

@RestController

@RequestMapping("/api/maintAuditTypeDesc")
public class MaintAuditTypeDescController {




	/**
	 * The maint audit type desc service is autowired and make methods available
	 * from service layer .
	 */
	@Autowired
	MaintAuditTypeDescService maintAuditTypeDescService;
	
	/**
	 * The business date service is autowired and make methods available from
	 * service layer .
	 */
	@Autowired
	BusinessDateService businessDateService;
	
	/**
	 * The sequence appender service is autowired and make methods available
	 * from service layer .
	 */
	@Autowired 
	SequenceAppenderService sequenceAppenderService;
	
	/** The Constant logger is used to specify the . */
	static final Logger logger = Logger.getLogger(MaintAuditTypeDescController.class);
	
	/**
	 * This method is use to Creates the maint audit type desc.
	 *
	 * @param maintAuditTypeDesc
	 *            specify the maint audit type desc
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(method=RequestMethod.POST,consumes=MediaType.APPLICATION_JSON_VALUE,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus createMaintAuditTypeDesc(@RequestBody MaintAuditTypeDesc maintAuditTypeDesc){
		ServiceStatus serviceStatus=new ServiceStatus();
		
		try {
			if(maintAuditTypeDesc!=null
					&& maintAuditTypeDesc.getLegalEntityCode()!=null
					&& !BankAuditUtil.isEmptyString(maintAuditTypeDesc.getAuditTypeCode())
					&& !BankAuditUtil.isEmptyString(maintAuditTypeDesc.getAuditTypeDesc())
					&& !BankAuditUtil.isEmptyString(maintAuditTypeDesc.getMaker())
	    		){ 
					//maintAuditTypeDesc.setAuditTypeCode("ATC"+sequenceAppenderService.getAutoSequenceId());
					maintAuditTypeDesc.setStatus("A");
					maintAuditTypeDesc.setBusinessdateTimestamp(businessDateService.getBusinessDateTimestampByLegalEntityCode(maintAuditTypeDesc.getLegalEntityCode()));
					maintAuditTypeDesc.setSysdateTimestamp(new Timestamp(System.currentTimeMillis()));
					maintAuditTypeDesc.setMakerTimestamp(maintAuditTypeDesc.getBusinessdateTimestamp());
				    maintAuditTypeDescService.createMaintAuditTypeDesc(maintAuditTypeDesc);
				    
				    serviceStatus.setStatus("success");
				    serviceStatus.setMessage("successfully created");
			}else {
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("invalid payload ");
			}
		}catch (Exception e) {
			logger.info("Inside exception createMaintAuditTypeDesc ......");
			serviceStatus.setStatus("failure");
			if(e instanceof org.springframework.dao.DataIntegrityViolationException) {
				serviceStatus.setMessage("DATAINTGRTY_VIOLATION");
			}else serviceStatus.setMessage("EXCP_OCCURED");
		}
		
		return serviceStatus;
	}
	
	/**
	 * This method is use to Update maint audit type desc.
	 *
	 * @param maintAuditTypeDesc
	 *            specify the maint audit type desc
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(method=RequestMethod.PUT,consumes=MediaType.APPLICATION_JSON_VALUE,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus updateMaintAuditTypeDesc(@RequestBody MaintAuditTypeDesc maintAuditTypeDesc){
		ServiceStatus serviceStatus=new ServiceStatus();
		
		if(maintAuditTypeDesc!=null
				&& maintAuditTypeDesc.getLegalEntityCode()!=null
				&& !BankAuditUtil.isEmptyString(maintAuditTypeDesc.getAuditTypeDesc())
				&& !BankAuditUtil.isEmptyString(maintAuditTypeDesc.getAuditTypeCode())
				&& !BankAuditUtil.isEmptyString(maintAuditTypeDesc.getMaker())
				
				){ 
		
			try {
				maintAuditTypeDesc.setStatus("A");
				maintAuditTypeDesc.setBusinessdateTimestamp(businessDateService.getBusinessDateTimestampByLegalEntityCode(maintAuditTypeDesc.getLegalEntityCode()));
				maintAuditTypeDesc.setSysdateTimestamp(new Timestamp(System.currentTimeMillis()));
				if(maintAuditTypeDesc.getStatus().equals(BankAuditConstant.STATUS_AUTH)){
					maintAuditTypeDesc.setCheckerTimestamp(businessDateService.getBusinessDateTimestampByLegalEntityCode(maintAuditTypeDesc.getLegalEntityCode()));
				}else{
					maintAuditTypeDesc.setMakerTimestamp(businessDateService.getBusinessDateTimestampByLegalEntityCode(maintAuditTypeDesc.getLegalEntityCode()));
				}
				
				    maintAuditTypeDescService.updateMaintAuditTypeDesc(maintAuditTypeDesc);
				    
				    serviceStatus.setStatus("success");
				    serviceStatus.setMessage("successfully updated");
			} catch (Exception e) {

				e.printStackTrace();
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("failure");
				
				if(e instanceof org.springframework.dao.DataIntegrityViolationException) {
					serviceStatus.setMessage("DATAINTGRTY_VIOLATION");
				}
				
			}
			
		}else {
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("invalid payload ");
		}
		
		return serviceStatus;
	}
	
	/**
	 * This method is use to Gets the maint audit type desc.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param auditTypeCode
	 *            specify the audit type code
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(value="/getByLegalEntityCodeAndAuditTypeCode/{legalEntityCode}/{auditTypeCode}",method=RequestMethod.GET,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus getMaintAuditTypeDesc(@PathVariable("legalEntityCode")Integer legalEntityCode,
			@PathVariable("auditTypeCode") String auditTypeCode
			){
		ServiceStatus serviceStatus=new ServiceStatus();
		
		if(legalEntityCode!=null 
				&& !BankAuditUtil.isEmptyString(auditTypeCode)){
			
			try {
			    MaintAuditTypeDesc MaintAuditTypeDesc=maintAuditTypeDescService.getMaintAuditTypeDesc(legalEntityCode,auditTypeCode);
			    if(MaintAuditTypeDesc!=null){
			    	serviceStatus.setResult(MaintAuditTypeDesc);
				    serviceStatus.setStatus("success");
				    serviceStatus.setMessage("successfully retrieved ");	
			    }else {
					serviceStatus.setStatus("failure");
					serviceStatus.setMessage("MaintAuditTypeDesc not found");
				}
				
			} catch (Exception e) {
				e.printStackTrace();
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("failure");
			}
			
		}else {
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("invalid payload ");
		}
		
		return serviceStatus;
	}
	
	
	/**
	 * This method is use to Gets the maint audit type desc.
	 *
	 * @param drawStr
	 *            specify the draw str
	 * @param length
	 *            specify the length
	 * @param start
	 *            specify the start
	 * @param search
	 *            specify the search
	 * @param orderColumnStr
	 *            specify the order column str
	 * @param orderDirection
	 *            specify the order direction
	 * @param legalEntityCodeStr
	 *            specify the legal entity code str
	 * @return the data table response class object with response status and
	 *         payload .
	 */
	
@RequestMapping(value="/getAll",method=RequestMethod.GET,produces=MediaType.APPLICATION_JSON_VALUE) 
	DataTableResponse getMaintAuditTypeDesc(
			@RequestParam("draw") String drawStr,
			@RequestParam("length") String length,
			@RequestParam("start") String start,
			@RequestParam("search[value]") String search,
			@RequestParam("order[0][column]") String orderColumnStr,
			@RequestParam("order[0][dir]")String orderDirection,
			@RequestParam("legalEntityCode")String legalEntityCodeStr){
		
		DataTableResponse dataTableResponse=null;
		
		if(!BankAuditUtil.isEmptyString(drawStr)
				&& !BankAuditUtil.isEmptyString(length)
				&& !BankAuditUtil.isEmptyString(start)
				&& !BankAuditUtil.isEmptyString(legalEntityCodeStr)){
			try {
				Integer legalEntityCode=Integer.parseInt(legalEntityCodeStr);
				Integer draw=Integer.parseInt(drawStr);
				Integer size=Integer.parseInt(length);
				Integer page=Integer.parseInt(start)/size;
				Integer orderColumn=null;
				if(!BankAuditUtil.isEmptyString(orderColumnStr)){
					orderColumn=Integer.parseInt(orderColumnStr);
				}
				dataTableResponse = maintAuditTypeDescService.getMaintAuditTypeDesc(legalEntityCode,search,orderColumn,orderDirection,page,size);
				dataTableResponse.setDraw(draw);    
					
			} catch (Exception e) {
                dataTableResponse=new DataTableResponse();
                dataTableResponse.setError(e.getMessage());
				e.printStackTrace();
			}
		
		}else {
			dataTableResponse=new DataTableResponse();
			dataTableResponse.setError("invalid data");
		}
		
		return dataTableResponse;
	}
	
	
	/**
	 * This method is use to Gets the maint audit type desc by legal entity
	 * code.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(value="/getByLegalEntityCode/{legalEntityCode}",method=RequestMethod.GET,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus getMaintAuditTypeDescByLegalEntityCode(@PathVariable("legalEntityCode")Integer legalEntityCode
			){
		ServiceStatus serviceStatus=new ServiceStatus();
		
		if(legalEntityCode!=null 
				){
			
			try {
			    List<MaintAuditTypeDesc> maintAuditTypeDescs=maintAuditTypeDescService.getMaintAuditTypeDescByLegalEntityCode(legalEntityCode);
			    if(maintAuditTypeDescs!=null&& !maintAuditTypeDescs.isEmpty()){
			    	serviceStatus.setResult(maintAuditTypeDescs);
				    serviceStatus.setStatus("success");
				    serviceStatus.setMessage("successfully retrieved ");	
			    }else {
					serviceStatus.setStatus("failure");
					serviceStatus.setMessage("MaintAuditTypeDescs not found");
				}
				
			} catch (Exception e) {
				e.printStackTrace();
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("failure");
			}
			
		}else {
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("invalid payload ");
		}
		
		return serviceStatus;
	}
	
	
@RequestMapping(value="/getByLegalEntityCodeAndUser/{legalEntityCode}/{userId}/{role}",method=RequestMethod.GET,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus getAuditTypeByLEAndUser(@PathVariable("legalEntityCode")Integer legalEntityCode,
			@PathVariable("userId")String userId,@PathVariable("role")String role
			){
		ServiceStatus serviceStatus=new ServiceStatus();
		
		if(legalEntityCode!=null && !BankAuditUtil.isEmptyString(userId) && !BankAuditUtil.isEmptyString(role)
				){
			
			try {
			    List<MaintAuditTypeDesc> maintAuditTypeDescs=maintAuditTypeDescService.getAuditTypeByLEAndUser(legalEntityCode,userId,role);
			    if(maintAuditTypeDescs!=null&& !maintAuditTypeDescs.isEmpty()){
			    	serviceStatus.setResult(maintAuditTypeDescs);
				    serviceStatus.setStatus("success");
				    serviceStatus.setMessage("successfully retrieved ");	
			    }else {
					serviceStatus.setStatus("failure");
					serviceStatus.setMessage("MaintAuditTypeDescs not found");
				}
				
			} catch (Exception e) {
				e.printStackTrace();
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("failure");
			}
			
		}else {
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("invalid payload ");
		}
		
		return serviceStatus;
	}
	

	
}
