/*
 * 
 */
package com.bankaudit.service;

import java.util.List;

import com.bankaudit.dto.DataTableResponse;
import com.bankaudit.model.MaintAuditActivity;

/**
 * The Interface {@link MaintAuditActivityService} define the methods that can
 * be exposed to controller layer in order to implements rest end-points and
 * define the business logic.
 *
 * @author amit.patel
 * @version 1.0
 */
public interface MaintAuditActivityService {

	/**
	 * This method is use to Creates the maint audit activity.
	 *
	 * @param maintAuditActivity
	 *            specify the maint audit activity
	 * @throws Exception
	 *             the exception
	 */
	void createMaintAuditActivity(MaintAuditActivity maintAuditActivity) throws Exception;

	/**
	 * This method is use to Update maint audit activity.
	 *
	 * @param maintAuditActivity
	 *            specify the maint audit activity
	 */
	void updateMaintAuditActivity(MaintAuditActivity maintAuditActivity);

	/**
	 * This method is use to Gets the maint audit activity.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param activityId
	 *            specify the activity id
	 * @param status
	 *            specify the status
	 * @return the maint audit activity .
	 */
	MaintAuditActivity getMaintAuditActivity(Integer legalEntityCode, String activityId,String status);

	

	/**
	 * This method is use to Gets the maint audit activities.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param search
	 *            specify the search
	 * @param orderColumn
	 *            specify the order column
	 * @param orderDirection
	 *            specify the order direction
	 * @param page
	 *            specify the page
	 * @param size
	 *            specify the size
	 * @return the data table response .
	 */
	DataTableResponse getMaintAuditActivities(Integer legalEntityCode, String search, Integer orderColumn,
			String orderDirection, Integer page, Integer size);

	/**
	 * This method is use to Gets the maint audit activity.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @return the list .
	 */
	List<MaintAuditActivity> getMaintAuditActivity(Integer legalEntityCode);

	/**
	 * This method is use to Checks if is maint audit activity.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param activityCode
	 *            specify the activity code
	 * @return the boolean .
	 */
	Boolean isMaintAuditActivity(Integer legalEntityCode, String activityCode);
	
	void deleteActivityByID(Integer legalEntityCode, String activityId, String userId);

}
