/*
 * 
 */
package com.bankaudit.service;

import java.util.List;

import com.bankaudit.dto.DataTableResponse;
import com.bankaudit.model.MaintAuditFinding;

/**
 * The Interface {@link MaintAuditFindingService} define the methods that can be
 * exposed to controller layer in order to implements rest end-points and define
 * the business logic.
 *
 * @author amit.patel
 * @version 1.0
 */
public interface MaintAuditFindingService {

	 /**
	 * This method is use to Creates the maint audit finding.
	 *
	 * @param maintAuditFinding
	 *            specify the maint audit finding
	 * @throws Exception
	 *             the exception
	 */
 	void createMaintAuditFinding(MaintAuditFinding maintAuditFinding) throws Exception;

	/**
	 * This method is use to Update maint audit finding.
	 *
	 * @param maintAuditFinding
	 *            specify the maint audit finding
	 */
	void updateMaintAuditFinding(MaintAuditFinding maintAuditFinding);

	/**
	 * This method is use to Gets the maint audit finding.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param findingCode
	 *            specify the finding code
	 * @param status
	 *            specify the status
	 * @return the maint audit finding .
	 */
	MaintAuditFinding getMaintAuditFinding(Integer legalEntityCode, String findingCode,String status);


	/**
	 * This method is use to Gets the maint audit findings.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param search
	 *            specify the search
	 * @param orderColumn
	 *            specify the order column
	 * @param orderDirection
	 *            specify the order direction
	 * @param page
	 *            specify the page
	 * @param size
	 *            specify the size
	 * @return the data table response .
	 */
	DataTableResponse getMaintAuditFindings(Integer legalEntityCode, String search, Integer orderColumn,
			String orderDirection, Integer page, Integer size);

	/**
	 * This method is use to Gets the maint audit finding.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @return the list .
	 */
	List<MaintAuditFinding> getMaintAuditFinding(Integer legalEntityCode);

	/**
	 * This method is use to Checks if is maint audit finding.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param findingCode
	 *            specify the finding code
	 * @return the boolean .
	 */
	Boolean isMaintAuditFinding(Integer legalEntityCode, String findingCode);
	
	void deleteFindingByID(Integer legalEntityCode, String findingId, String userId);

}
