/*
 * 
 */
package com.bankaudit.service;

import java.util.List;

import com.bankaudit.dto.DataTableResponse;
import com.bankaudit.model.MaintAuditProcess;

/**
 * The Interface {@link MaintAuditProcessService} define the methods that can be
 * exposed to controller layer in order to implements rest end-points and define
 * the business logic.
 *
 * @author amit.patel
 * @version 1.0
 */
public interface MaintAuditProcessService {

	/**
	 * This method is use to Creates the maint audit process.
	 *
	 * @param maintAuditProcess
	 *            specify the maint audit process
	 * @throws Exception
	 *             the exception
	 */
	void createMaintAuditProcess(MaintAuditProcess maintAuditProcess) throws Exception ;

	/**
	 * This method is use to Update maint audit process.
	 *
	 * @param maintAuditProcess
	 *            specify the maint audit process
	 */
	void updateMaintAuditProcess(MaintAuditProcess maintAuditProcess);

	/**
	 * This method is use to Gets the maint audit process.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param processCode
	 *            specify the process code
	 * @param status
	 *            specify the status
	 * @return the maint audit process .
	 */
	MaintAuditProcess getMaintAuditProcess(Integer legalEntityCode, String processCode, String status);


	/**
	 * This method is use to Gets the maint audit process.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param search
	 *            specify the search
	 * @param orderColumn
	 *            specify the order column
	 * @param orderDirection
	 *            specify the order direction
	 * @param page
	 *            specify the page
	 * @param size
	 *            specify the size
	 * @return the data table response .
	 */
	DataTableResponse getMaintAuditProcess(Integer legalEntityCode, String search, Integer orderColumn,
			String orderDirection, Integer page, Integer size);

	/**
	 * This method is use to Gets the maint audit process.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @return the list .
	 */
	List<MaintAuditProcess> getMaintAuditProcess(Integer legalEntityCode);

	/**
	 * This method is use to Checks if is maint audit process.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param findingCode
	 *            specify the finding code
	 * @return the boolean .
	 */
	Boolean isMaintAuditProcess(Integer legalEntityCode, String findingCode);
	
	void deleteProcessByID(Integer legalEntityCode, String processId, String userId);

}
