/*
 * 
 */
package com.bankaudit.service;

import java.util.List;

import com.bankaudit.dto.DataTableResponse;
import com.bankaudit.model.MaintAuditSubgroup;

/**
 * The Interface {@link MaintAuditSubgroupService} define the methods that can
 * be exposed to controller layer in order to implements rest end-points and
 * define the business logic.
 *
 * @author amit.patel
 * @version 1.0
 */
public interface MaintAuditSubgroupService {

	/**
	 * This method is use to Creates the maint audit subgroup.
	 *
	 * @param maintAuditSubgroup
	 *            specify the maint audit subgroup
	 * @throws Exception
	 *             the exception
	 */
	void createMaintAuditSubgroup(MaintAuditSubgroup maintAuditSubgroup)throws Exception;

	/**
	 * This method is use to Update maint audit subgroup.
	 *
	 * @param maintAuditSubgroup
	 *            specify the maint audit subgroup
	 */
	void updateMaintAuditSubgroup(MaintAuditSubgroup maintAuditSubgroup);

	/**
	 * This method is use to Gets the maint audit subgroup.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param auditSubGroupCode
	 *            specify the audit sub group code
	 * @param auditGroupCode
	 *            specify the audit group code
	 * @param auditTypeCode
	 *            specify the audit type code
	 * @param status
	 *            specify the status
	 * @return the maint audit subgroup .
	 */
	MaintAuditSubgroup getMaintAuditSubgroup(Integer legalEntityCode, String auditSubGroupCode, String auditGroupCode,
			String auditTypeCode,String status);


	/**
	 * This method is use to Gets the maint audit subgroupes.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param search
	 *            specify the search
	 * @param orderColumn
	 *            specify the order column
	 * @param orderDirection
	 *            specify the order direction
	 * @param page
	 *            specify the page
	 * @param size
	 *            specify the size
	 * @return the data table response .
	 */
	DataTableResponse getMaintAuditSubgroupes(Integer legalEntityCode, String search, Integer orderColumn,
			String orderDirection, Integer page, Integer size);

	/**
	 * This method is use to Gets the maint audit subgroupes.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param auditTypeCode
	 *            specify the audit type code
	 * @param auditGroupCode
	 *            specify the audit group code
	 * @param auditSubGroupCode
	 *            specify the audit sub group code
	 * @return the list .
	 */
	List<MaintAuditSubgroup> getMaintAuditSubgroupes(Integer legalEntityCode, String auditTypeCode,
			String auditGroupCode, String auditSubGroupCode);

	/**
	 * This method is use to Gets the maint audit subgroupe.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @return the list .
	 */
	List<MaintAuditSubgroup> getMaintAuditSubgroupe(Integer legalEntityCode);

	/**
	 * This method is use to Checks if is maint audit subgroup.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param auditTypeCode
	 *            specify the audit type code
	 * @param auditGroupCode
	 *            specify the audit group code
	 * @param auditSubGroupCode
	 *            specify the audit sub group code
	 * @return the boolean .
	 */
	Boolean isMaintAuditSubgroup(Integer legalEntityCode, String auditTypeCode, String auditGroupCode,
			String auditSubGroupCode);
	
	Boolean isMaintAuditSubgroupLE(Integer legalEntityCode, String auditSubGroupCode);
	
	void deleteAuditSubGroupByID(Integer legalEntityCode, String auditTypeCode, String auditGroupCode, String auditSubGroupCode, String userId);

}
